@@ -8,127 +8,44 @@ from cherrypy import _cperror, wsgiserver
 from cherrypy.lib import http as _http
 
 
-#                            Internal Redirect                            #
-
-
-class InternalRedirector(object):
-    """WSGI middleware which handles cherrypy.InternalRedirect.
-    
-    When cherrypy.InternalRedirect is raised, this middleware traps it,
-    rewrites the WSGI environ using the new path and query_string,
-    and calls the next application again. Because the wsgi.input stream
-    may have already been consumed by the next application, the redirected
-    call will always be of HTTP method "GET", and therefore any params must
-    be passed in the InternalRedirect object's query_string attribute.
-    If you need something more complicated, make and raise your own
-    exception and your own WSGI middleware to trap it. ;)
-    
-    It would be a bad idea to raise InternalRedirect after you've already
-    yielded response content, although an enterprising soul could choose
-    to abuse this.
-    
-    nextapp: the next application callable in the WSGI chain.
-    
-    recursive: if False (the default), each URL (path + qs) will be
-    stored, and, if the same URL is requested again, RuntimeError will
-    be raised. If 'recursive' is True, no such error will be raised.
-    """
-    
-    def __init__(self, nextapp, recursive=False):
-        self.nextapp = nextapp
-        self.recursive = recursive
-    
-    def __call__(self, environ, start_response):
-        return IRResponse(self.nextapp, environ, start_response, self.recursive)
-
-
-class IRResponse(object):
-    
-    def __init__(self, nextapp, environ, start_response, recursive):
-        self.redirections = []
-        self.recursive = recursive
-        self.environ = environ.copy()
-        self.nextapp = nextapp
-        self.start_response = start_response
-        self.response = None
-        self.iter_response = None
-        self.setapp()
-    
-    def setapp(self):
-        while True:
-            try:
-                self.response = self.nextapp(self.environ, self.start_response)
-                self.iter_response = iter(self.response)
-                return
-            except _cherrypy.InternalRedirect, ir:
-                self.close()
-                self.setenv(ir)
-    
-    def setenv(self, ir):
-        env = self.environ
-        if not self.recursive:
-            if ir.path in self.redirections:
-                raise RuntimeError("InternalRedirector visited the "
-                                   "same URL twice: %r" % ir.path)
-            else:
-                # Add the *previous* path_info + qs to redirections.
-                sn = env.get('SCRIPT_NAME', '')
-                path = env.get('PATH_INFO', '')
-                qs = env.get('QUERY_STRING', '')
-                if qs:
-                    qs = "?" + qs
-                self.redirections.append(sn + path + qs)
-        
-        # Munge environment and try again.
-        env['REQUEST_METHOD'] = "GET"
-        env['PATH_INFO'] = ir.path
-        env['QUERY_STRING'] = ir.query_string
-        env['wsgi.input'] = _StringIO.StringIO()
-        env['CONTENT_LENGTH'] = "0"
-    
-    def close(self):
-        if hasattr(self.response, "close"):
-            self.response.close()
-    
-    def __iter__(self):
-        return self
-    
-    def next(self):
-        while True:
-            try:
-                return self.iter_response.next()
-            except _cherrypy.InternalRedirect, ir:
-                self.close()
-                self.setenv(ir)
-                self.setapp()
-
-
-
 #                           WSGI-to-CP Adapter                           #
 
 
 class AppResponse(object):
+    """A WSGI application for CherryPy Application responses.
     
-    throws = (KeyboardInterrupt, SystemExit, _cherrypy.InternalRedirect)
+    recursive: if False (the default), each URL (path + qs) will be stored,
+    and, if the same URL is requested again via an InternalRedirect,
+    RuntimeError will be raised. If 'recursive' is True, no such error
+    will be raised.
+    """
+    
+    throws = (KeyboardInterrupt, SystemExit)
     request = None
     
-    def __init__(self, environ, start_response, cpapp):
+    def __init__(self, environ, start_response, cpapp, recursive=False):
+        self.redirections = []
+        self.recursive = recursive
+        self.environ = environ
+        self.start_response = start_response
+        self.cpapp = cpapp
+        self.setapp()
+    
+    def setapp(self):
+        # Stage 1: by whatever means necessary, obtain a status, header
+        #          set and body, with an optional exception object.
         try:
-            self.request = self.get_engine_request(environ, cpapp)
-            
-            meth = environ['REQUEST_METHOD']
-            path = environ.get('SCRIPT_NAME', '') + environ.get('PATH_INFO', '')
-            qs = environ.get('QUERY_STRING', '')
-            rproto = environ.get('SERVER_PROTOCOL')
-            headers = self.translate_headers(environ)
-            rfile = environ['wsgi.input']
-            
-            response = self.request.run(meth, path, qs, rproto, headers, rfile)
-            s, h, b = response.status, response.header_list, response.body
+            self.request = self.get_engine_request()
+            s, h, b = self.get_response()
             exc = None
         except self.throws:
             self.close()
             raise
+        except _cherrypy.InternalRedirect, ir:
+            self.environ['cherrypy.previous_request'] = _cherrypy._serving.request
+            self.close()
+            self.iredirect(ir.path, ir.query_string)
+            return
         except:
             if getattr(self.request, "throw_errors", False):
                 self.close()
@@ -143,8 +60,10 @@ class AppResponse(object):
         
         self.iter_response = iter(b)
         
+        # Stage 2: now that we have a status, header set, and body,
+        #          finish the WSGI conversation by calling start_response.
         try:
-            start_response(s, h, exc)
+            self.start_response(s, h, exc)
         except self.throws:
             self.close()
             raise
@@ -162,6 +81,50 @@ class AppResponse(object):
             s, h, b = _cperror.bare_error()
             self.iter_response = iter(b)
     
+    def iredirect(self, path, query_string):
+        """Doctor self.environ and perform an internal redirect.
+        
+        When cherrypy.InternalRedirect is raised, this method is called.
+        It rewrites the WSGI environ using the new path and query_string,
+        and calls a new CherryPy Request object. Because the wsgi.input
+        stream may have already been consumed by the next application,
+        the redirected call will always be of HTTP method "GET"; therefore,
+        any params must be passed in the query_string argument, which is
+        formed from InternalRedirect.query_string when using that exception.
+        If you need something more complicated, make and raise your own
+        exception and write your own AppResponse subclass to trap it. ;)
+        
+        It would be a bad idea to redirect after you've already yielded
+        response content, although an enterprising soul could choose
+        to abuse this.
+        """
+        env = self.environ
+        if not self.recursive:
+            sn = env.get('SCRIPT_NAME', '')
+            qs = query_string
+            if qs:
+                qs = "?" + qs
+            if sn + path + qs in self.redirections:
+                raise RuntimeError("InternalRedirector visited the "
+                                   "same URL twice: %r + %r + %r" %
+                                   (sn, path, qs))
+            else:
+                # Add the *previous* path_info + qs to redirections.
+                p = env.get('PATH_INFO', '')
+                qs = env.get('QUERY_STRING', '')
+                if qs:
+                    qs = "?" + qs
+                self.redirections.append(sn + p + qs)
+        
+        # Munge environment and try again.
+        env['REQUEST_METHOD'] = "GET"
+        env['PATH_INFO'] = path
+        env['QUERY_STRING'] = query_string
+        env['wsgi.input'] = _StringIO.StringIO()
+        env['CONTENT_LENGTH'] = "0"
+        
+        self.setapp()
+    
     def __iter__(self):
         return self
     
@@ -175,11 +138,17 @@ class AppResponse(object):
                 chunk = chunk.encode("ISO-8859-1")
             return chunk
         except self.throws:
+            self.close()
             raise
+        except _cherrypy.InternalRedirect, ir:
+            self.environ['cherrypy.previous_request'] = _cherrypy._serving.request
+            self.close()
+            self.iredirect(ir.path, ir.query_string)
         except StopIteration:
             raise
         except:
             if getattr(self.request, "throw_errors", False):
+                self.close()
                 raise
             
             _cherrypy.log(traceback=True)
@@ -192,15 +161,23 @@ class AppResponse(object):
             return "".join(b)
     
     def close(self):
-        if hasattr(self.request, "close"):
-            try:
-                self.request.close()
-            except:
-                _cherrypy.log(traceback=True)
+        _cherrypy.engine.release()
     
-    def get_engine_request(self, environ, cpapp):
+    def get_response(self):
+        """Grab a request object from the engine and return its response."""
+        meth = self.environ['REQUEST_METHOD']
+        path = (self.environ.get('SCRIPT_NAME', '') +
+                self.environ.get('PATH_INFO', ''))
+        qs = self.environ.get('QUERY_STRING', '')
+        rproto = self.environ.get('SERVER_PROTOCOL')
+        headers = self.translate_headers(self.environ)
+        rfile = self.environ['wsgi.input']
+        response = self.request.run(meth, path, qs, rproto, headers, rfile)
+        return response.status, response.header_list, response.body
+    
+    def get_engine_request(self):
         """Return a Request object from the CherryPy Engine using environ."""
-        env = environ.get
+        env = self.environ.get
         
         local = _http.Host('', int(env('SERVER_PORT', 80)),
                            env('SERVER_NAME', ''))
@@ -215,12 +192,11 @@ class AppResponse(object):
         # user after having been mapped to a local account.
         # Both IIS and Apache set REMOTE_USER, when possible.
         request.login = env('LOGON_USER') or env('REMOTE_USER') or None
-        request.multithread = environ['wsgi.multithread']
-        request.multiprocess = environ['wsgi.multiprocess']
-        request.wsgi_environ = environ
-        request.app = cpapp
-        request.prev = env('cherrypy.request')
-        environ['cherrypy.request'] = request
+        request.multithread = self.environ['wsgi.multithread']
+        request.multiprocess = self.environ['wsgi.multiprocess']
+        request.wsgi_environ = self.environ
+        request.app = self.cpapp
+        request.prev = env('cherrypy.previous_request', None)
         return request
     
     headerNames = {'HTTP_CGI_AUTHORIZATION': 'Authorization',
@@ -260,7 +236,7 @@ class CPWSGIApp(object):
         named WSGI callable (from the pipeline) as keyword arguments.
     """
     
-    pipeline = [('iredir', InternalRedirector)]
+    pipeline = []
     head = None
     config = {}
     
