@@ -33,5 +33,9 @@ if __name__ == '__main__':
             '/unknown': {'toobles.gzip.on': True},
             # Warn special on cherrypy.<known ns>.*
             '/cpknown': {'cherrypy.tools.encode.on': True},
+            # Warn on mismatched types
+            '/conftype': {'request.show_tracebacks': 14},
+            # Warn on unknown tool.
+            '/web': {'tools.unknown.on': True},
             }
     cherrypy.quickstart(Root(), config=conf)
